/*
* Distributed Computational System
* Standalone Program to Find Prime Numbers
* (Used to compare difference between distributed system and
* single system calculating prime numbers)
*
* $Id: find_primes.c,v 1.4 2001/11/30 23:09:50 kris Exp $
*/

#include <math.h>	/* for sqrt */
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include "cp_precalc.h"

int isprime( unsigned long num );
int usec( void );

int main( int argc, char **argv )
{
	unsigned long min, max, i;
	int n = 0, start, stop;
	double sec, tt;
	FILE *tmp;

	if( argc < 3 ) {
		fprintf( stderr, "Usage: %s <min> <max>\n", argv[0] );
		return 1;
	}

	min = (unsigned long)atoi( argv[1] );
	max = (unsigned long)atoi( argv[2] );

	if( min > max ) {
		fprintf( stderr, "Min > Max!\n" );
		return 1;
	}

	tmp = fopen( "findprimes.out", "w" );

	start = usec();
	for( i = min; i <= max; i++ ) {
		if( isprime( i ) ) {
			fprintf( tmp, "%d\n", i );
			n++;
		}
	}
	stop = usec();

	fclose( tmp );

	printf( "Found %d primes between %d and %d (see findprimes.out)\n", n, min, max );
	tt = stop - start;
	sec = (double)tt / 1000000;
	printf( "Total running time: %f seconds for range size %d\n", sec, max - min );

	return 0;
}

int isprime( unsigned long num )
{
	int i;

	for( i = 0; (g_precalc[i] < (unsigned long)sqrt( (double)num ) + 1) && (g_precalc[i] != 0); i++ ) {
		if( num % g_precalc[i] == 0 )
			return 0;
	}
	return 1;
}

int usec( void ) {
	struct timeval tv;
	struct timezone tz;

	gettimeofday( &tv, &tz );
	return (tv.tv_sec & 2047) * 1000000 + tv.tv_usec;
}
