/*
* Distributed Computational System
* Protocol Layer
*
* $Id: protocol.c,v 1.5 2001/11/29 07:06:17 kris Exp $
*/

#include <errno.h>
#include "protocol.h"
#include "port.h"

#ifdef _WIN32
	#include <winsock2.h>
	#include <windows.h>
	#include <io.h>
#else
	extern int errno;
#endif

/* a happy fun static buffer */
packet_buf_t g_send_buf = {0, 0, 0, 0, 0, 0, 0};

int PR_Send( int socket, unsigned short flags, command_t *command )
{
	/* in this prototype, we'll ignore flags */

	g_send_buf.cmd = command->cmd;
	g_send_buf.len = command->len;
	g_send_buf.data = command->data;
	g_send_buf.checksum = 0;

	if( write( socket, &g_send_buf.version, 12 ) < 0 )
		return 1;
	if( write( socket, g_send_buf.data, g_send_buf.len ) < 0 )
		return 1;
	return 0;
}

int PR_Recv( int socket, packet_buf_t *packet_buf )
{
	void *i;
	int n, max;

	do {
		if( packet_buf->bytes_read < 12 ) {
			i = &(packet_buf->version) + packet_buf->bytes_read;
			max = 12 - packet_buf->bytes_read;
		} else {
			i = packet_buf->data + packet_buf->bytes_read - 12;
			max = packet_buf->len - (packet_buf->bytes_read - 12);
		}
		n = read( socket, i, max );
		if( n < 0 ) {
			if( ERRNO == EINTR )
				continue;
			else if( ERRNO == EAGAIN )
				return 1;
			else
				return -1;
		}
		if( n == 0 )
			return 0;
		packet_buf->bytes_read += n;
		if( packet_buf->bytes_read >= 12 && (packet_buf->bytes_read - 12 == (unsigned long)packet_buf->len) )
			return 2;
		if( packet_buf->bytes_read == 12 ) {
			if( packet_buf->len == 0 )
				return 2;
			packet_buf->data = (unsigned char *)malloc( packet_buf->len );
		}
	} while( 1 );
	/* we should never get here */
	return -1;
}

int PR_RecvBlocking( int socket, command_t *cmd )
{
	packet_buf_t packet_buf;
	int n;

	INIT_PACKET_BUF(packet_buf);
	n = read( socket, &(packet_buf.version), 12 );
	if( n != 12 )
		return -1;
	cmd->cmd = packet_buf.cmd;
	cmd->len = packet_buf.len;
	if( packet_buf.len == 0 ) {
		cmd->data = 0;
		return 2;
	}
	packet_buf.data = (unsigned char *)malloc( packet_buf.len );
	n = read( socket, packet_buf.data, packet_buf.len );
	if( n != packet_buf.len ) {
		FREE_PACKET_DATA(packet_buf);
		return -1;
	}
	cmd->data = packet_buf.data;
	return 2;
}
