/*
* Distributed Computational System
* Prime Number Calculator - Server Plugin
*
* $Id: sp_primes.c,v 1.6 2001/11/29 23:33:34 kris Exp $
*/

#include "server_plugin.h"
#include "db_proto.h"
#include <stdio.h>	/* FILE* stuff */
#include <string.h>	/* for memcpy */

/*
	we'll ask each client to calculate all the primes between
	g_current_index and g_current_index + CHUNK_RANGE_SIZE.
*/
#define CHUNK_RANGE_SIZE 2000000
#define MAX_RANGE_NUM 1000000000
#define PRINT_FILE "sp_primes.txt.out"
#define BIN_FILE "sp_primes.bin.out"

int g_plugin_id = -1;
unsigned long g_current_index = 0;
int g_cur_chunk_id = 0;
FILE *g_output_print, *g_output_bin;

int SP_Init( int plugin_id )
{
	g_plugin_id = plugin_id;

	/*
		we never call close on this, because it will get
		closed when we terminate the server.  however, since
		in a non-prototype we might want to be able to unload
		plugins, it might be useful to add an SP_Cleanup
		function to server plugins (and something similar to
		client plugins) so that we can do this.

		for demonstration purposes, we'll output to both a
		binary file (4 bytes per number), and a text file
		(each number on a line, in ascii).
	*/
	g_output_print = fopen( PRINT_FILE, "w+" );
	g_output_bin = fopen( BIN_FILE, "w+" );
	return 0;
}

int SP_GetVersion()
{
	return 0;
}

int SP_GetProblemChunk( server_chunk_t *chunk )
{
	old_chunk_t old_chunk;

	/*
		set and increment our chunk id.  in this prototype we
		can just do this in the plugin, since there is only
		one plugin/one server.  in a real system, this would
		be a guid.
	*/
	memset( chunk->chunk_id, 0, 16 );
	memcpy( chunk->chunk_id, &g_cur_chunk_id, 4 );
	memcpy( old_chunk.chunk_id, chunk->chunk_id, 16 );
	g_cur_chunk_id++;

	chunk->chunk_type = 0;	/* this is ignored in the prototype */
	chunk->problem_id = 0;	/* this too */
	chunk->len = 8;		/* 8 bytes of data (a min/max) */
	chunk->data = (unsigned char *)malloc( chunk->len );

	if( g_current_index >= MAX_RANGE_NUM ) {
		/* all we have left to complete are old chunks */
		if( DBP_PopOldChunk( &old_chunk ) < 0 )
			return -1;
		memcpy( chunk->data, &old_chunk.min, 4 );
		memcpy( chunk->data + 4, &old_chunk.max, 4 );
		DBP_PushOldChunk( &old_chunk );
		fprintf( stderr, "Assigning range: %d to %d\n", old_chunk.min, old_chunk.max );
		return 0;
	}

	/*
		IMPORTANT: chunk->data is a statically allocated array
		that is provided by the server module; in a real-world
		situation we might have to do it dynamically because
		we don't know the max amount of chunk data that any
		plugin would need to send.
	*/
	memcpy( chunk->data, &g_current_index, 4 );
	memcpy( &old_chunk.min, &g_current_index, 4 );
	g_current_index += CHUNK_RANGE_SIZE;
	if( g_current_index > MAX_RANGE_NUM )
		g_current_index = MAX_RANGE_NUM;
	memcpy( chunk->data + 4, &g_current_index, 4 );
	memcpy( &old_chunk.max, &g_current_index, 4 );
	DBP_PushOldChunk( &old_chunk );
	fprintf( stderr, "Assigning range: %d to %d\n", old_chunk.min, old_chunk.max );

	return 0;
}

int SP_CompletedChunk( server_chunk_t *chunk )
{
	int i;

#define GETNUMBER(i) (*(unsigned long *)(chunk->data + (i)))

	/*
		if this chunk_id isn't in our queue of assigned chunks,
		discard the results (it's already been returned by
		someone else).
	*/
	if( DBP_RemoveOldChunk( chunk->chunk_id ) < 0 )
		return 0;

	fprintf( stderr, "Completed chunk returned: %d primes\n", chunk->len / 4 );
	for( i = 0; i < chunk->len; i += 4 )
		fprintf( g_output_print, "%d\n", GETNUMBER(i) );
	fwrite( chunk->data, 4, chunk->len / 4, g_output_bin );
	fflush( g_output_print );
	fflush( g_output_bin );

	return 0;
}
